//---------------------------------------------------------------------------
//
// Evj        Model the evolution of DNA binding site recognition
//
// Module:    DataStatPanel, a Swing panel that displays data & statistics.
// Author:    Paul C. Anagnostopoulos
// Created:   29 November 2005
//
// This computer program fits the description in the U.S. Copyright Act
// of a "United States Government work." It was written as a part of
// my/our official duties as a Government employee. This means it cannot
// be copyrighted. The program is freely available for use without a
// copyright notice, and there are no restrictions on its use, now or
// subsequently. We retain no rights in the program. Please acknowledge
// the source of this code in any releases you make.
//
//---------------------------------------------------------------------------


import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.lang.Math;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;

//                      Data & Statistics Panel
//                      ---- - ---------- -----


public final class DataStatPanel extends JPanel implements NexusObserver
{
  private static final int topBaseline = 30;
  
  private static final int column1X = 10,
                           column2X = 130;
  
  private static final int dataWidth = 10;
  
  private static final Font textFont = new Font("SansSerif", Font.BOLD, 14);
  private static final Font dataFont = new Font("Monospaced", Font.PLAIN, 14);
  
  private static final DecimalFormat
    format0_0000  = new DecimalFormat("0.0000"),
    dataFormat    = new DecimalFormat("'+'0;-0"),
    integerFormat = new DecimalFormat("#,##0");
  
  private JTextArea thresholdText;
  
  private JScrollPane scrollPane;
  private Nexus acceptedNexus = null;

  // The constructor saves the parent scroll pane.

  public DataStatPanel (JScrollPane sp)
  {
    super();
    scrollPane = sp;
    fixDataFont();
    createComponents(this);
  }

  private void fixDataFont ()
  {
  }
  
  private void createComponents (Container cont)
  {
    thresholdText = new JTextArea();
    thresholdText.setEditable(false);
    thresholdText.setFont(dataFont);
    thresholdText.setToolTipText("This is the tooltip.");
///    thresholdText.setBounds(100, 100, 100, 100);
    cont.add(thresholdText);
  }
  
  // This public method implements the NexusObserver interface. It
  // simply saves the supplied nexus and repaints the display.

  public void acceptNexus (Nexus nexus)
  {
    acceptedNexus = nexus;
    repaint();
  }

//                      Paint Component
//                      ----- ---------


  private Graphics2D g2;
  int y;

  public void paintComponent (Graphics g)
  {
    super.paintComponent(g);

    // Pick up the nexus so we are impervious to nexuses coming in
    // while drawing. If it is null, forget it.

    Nexus nexus = acceptedNexus;
    if (nexus == null) return;
    g2 = (Graphics2D)g;

    int scrollWidth = scrollPane.getViewport().getExtentSize().width;

    y = topBaseline;
    g2.setFont(textFont);

    display("Population", nexus.population, " creatures");
    if (nexus.useFounder)
      display("Founders", nexus.founders, " creatures");
    else
      display("Founders", "(not using founder model)");

    display("Potential sites", nexus.potentialSites,
            " on 1 chromosome of length ", nexus.chromosome.length, " bases");
    display("Chromosome", "weight matrix position ", nexus.genePos+1,
                          "; threshold position ", nexus.thresholdPos+1,
                          "; available position ", nexus.availablePos+1);
    
    display("Binding sites", nexus.sites,
                             " sites; width ", nexus.siteWidth,
                             " bases; information capacity ",
                               2 * nexus.siteWidth,
                             " bits (Rcapacity)");
    
    display("Mutations", nexus.mutations, " per creature per generation");

    display();

    display("Generation", nexus.generation,
            " of ", nexus.cycles, " to be run");

    display("Births", nexus.births);
    
    display("Deaths", "total ", nexus.deaths,
                      "; by selection ", nexus.selectionDeaths,
                      " (more mistakes than the median score [least-fit surviving creature])");
    
    display();

    display("Mean Age", nexus.generation > 0 ?
                        (int)Math.round(nexus.bestTotalAge / nexus.generation)
                        : 0);
    
    thresholdText.setText(formatData(nexus.threshold));
    display("Threshold", thresholdText, 80, 20);
    
    g2.drawString("This display is not yet complete.",
                  column2X, y);

    setPreferredSize(new Dimension(scrollWidth, 500));
    revalidate();
  }

//                      Display Methods
//                      ------- -------


  private void display (String col1, String col2)
  {
    g2.drawString(col1, column1X, y);
    g2.drawString(col2, column2X, y);
    y += 24;
  }
  
  private void display ()
  {
    display("", "");
  }
  
  private void display (String title, int value1)
  {
    display(title,
            formatInteger(value1));
  }
  
  private void display (String title, int value1, String phrase1)
  {
    display(title,
            formatInteger(value1) + phrase1);
  }

  private void display (String title, double value1, String phrase1)
  {
    display(title,
            formatReal(value1) + phrase1);
  }

  private void display (String title, int value1,
                        String phrase1, int value2, String phrase2)
  {
    display(title,
            formatInteger(value1) + phrase1 +
            formatInteger(value2) + phrase2);
  }

  private void display (String title, int value1,
                        String phrase1, int value2, String phrase2,
                        int value3, String phrase3)
  {
    display(title,
            formatInteger(value1) + phrase1 +
            formatInteger(value2) + phrase2 +
            formatInteger(value3) + phrase3);
  }

  private void display (String title, String phrase1, int value1,
                                      String phrase2, int value2)
  {
    display(title,
            phrase1 + formatInteger(value1) +
            phrase2 + formatInteger(value2));
  }

  private void display (String title, String phrase1, int value1,
                                      String phrase2, int value2,
                                      String phrase3)
  {
    display(title,
            phrase1 + formatInteger(value1) +
            phrase2 + formatInteger(value2) +
            phrase3);
  }

  private void display (String title, String phrase1, int value1,
                                      String phrase2, int value2,
                                      String phrase3, int value3)
  {
    display(title,
            phrase1 + formatInteger(value1) +
            phrase2 + formatInteger(value2) +
            phrase3 + formatInteger(value3));
  }

  private void display (String col1, JTextArea col2, int width, int height)
  {
    col2.setBounds(column2X, y-10, width, height);
    g2.drawString(col1, column1X, y);
    col2.revalidate();
    y += col2.getHeight() + 10;
  }

//                      Utilities
//                      ---------


  private String formatData (int n)
  {
    String result = dataFormat.format(n);

    return "          ".substring(0, dataWidth - result.length()) + result;
  }
  
  private String formatInteger (int n)
  {
    return integerFormat.format(n);
  }

  private String formatReal (double r)
  {
    return format0_0000.format(r);
  }
}
